function [xmin,q,solutionStatus,noIntegerVar] = ilppglpkreadres_zig(filename,filenameRes,filenameLPModel)
%The function reads files produced by GLPK solver and returns result of
%optimization, i.e. vector with optimal solution. The vector must be
%contained in the generated file in the following form:
%
% Result of scheduling:
%
%
%  printf"Result of scheduling:\n";
%  printf{i in T}: "%d\n", s[i];
%
% PARAMETERS:
%   filename    file with results
%   filenameRes	file with status of the solution
%
% Copyright Sucha Hanzalek {suchap,hanzalek}@fel.cvut.cz
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

noIntegerVar = 0;       %Number of integer variables in ILP model

%%%%%%%%%% Count number of integer variables %%%%%%%%%%
fid = fopen(filenameLPModel,'r');
count = 0;

while(1)
    line = fgetl(fid);
    if(line == -1)
        break;
    end
    if(strcmp(line,'Generals'))
        count = 1;
        continue;
    end
    
    if(strcmp(line,''))
        count = 0;
        continue;
    end
    
    if(count == 1)
        noIntegerVar = noIntegerVar + 1;
    end
        
end

fclose(fid);


%%%%%%%%%%%%% Read status of the solution %%%%%%%%%%%%
fidRes = fopen(filenameRes,'r');
if(fidRes == -1)
    error(['Can not read file ''' filenameRes '''']);
end

solutionStatus = 0;     %Status of the solution 0=nonfeasible, 1=optimal, 2=feasible but not optimal

for(i=1:5)
    tline = fgetl(fidRes);
end
if ~ischar(tline)
    error('GPR:ilppglpkreadres',['Incorrect file ''' filenameRes '''']);
end
if(strcmp(tline,'Status:     INTEGER UNDEFINED') || strcmp(tline,'Status:     UNDEFINED') || strcmp(tline,'Status:     INTEGER EMPTY'))
    solutionStatus = 0;
    xmin = [];
    q = {};
    fclose(fidRes);
    return;
end
if(strcmp(tline,'Status:     INTEGER NON-OPTIMAL'))
    solutionStatus = 2;
end
if(strcmp(tline,'Status:     INTEGER OPTIMAL') || strcmp(tline,'Status:     OPTIMAL'))
    solutionStatus = 1;
end

fclose(fidRes);

if(solutionStatus == 0)
    error('GPR:ilppglpkreadres',['Incorrect file ''' filenameRes '''']);
end


%%%%%%%%%%% Read results %%%%%%%%%%%
fid = fopen(filename,'r');
if(fid == -1)
    error('GPR:ilppglpkreadres',['Can not read file ''' filename '''']);
end

startReading = 0;
xmin = [];
q = [];
%y = [];

while 1
    tline = fgetl(fid);
    if ~ischar(tline), break, end
    if(strcmp(tline,'PROBLEM HAS NO FEASIBLE SOLUTION'))
        fclose(fid);
        error('GPR:ilppglpkreadres',['A conflict between ' filenameRes ' and ' filename]);
    end
    if(strcmp(tline,'PROBLEM HAS NO INTEGER FEASIBLE SOLUTION'))
        fclose(fid);
        error('GPR:ilppglpkreadres',['A conflict between ' filenameRes ' and ' filename]);
    end
    if(strcmp(tline,'Start times:'))
        startReading = 1;
        continue;
    end
    %Read vector of start times
    if(startReading == 1)
        if(strcmp(tline,'Indeces of the execution period:'))
            startReading = 2;
            continue;
        end
%         if(strcmp(tline,'Conflicts between tasks:'))
%             startReading = 3;
%             continue;
%         end
        [x status] = str2num(tline);
        if(status == 0)
            break;
        end
        xmin(end+1) = x;
    end
    %Read vector of processor assignements
    if(startReading == 2)
        [x status] = str2num(tline);
        if(status == 0)
            break;
        end
        q(end+1) = x;
    end
    %Read matrix of r[i,j]
%     if(startReading == 3)
%         [x status] = str2num(tline);
%         if(status == 0)
%             break;
%         end
%         y = [y; x];
%     end
end

fclose(fid);

if(startReading == 0)
    error('GPR:ilppglpkreadres',['Model processing error. See ''' filename '''.']);
end

% if(startReading == 3)
%     procAssign = y + y';
% end

%end of file
