function [CTT,no_routers,I] = topology()
% TOPOLOGY description of the WSN cluster-tree topology
%
% PARAMETERS:
%   CTT     adjacency matrix - description of cluster-tree topology CTT
%   (logical topology) - parent-child relation
%   no_routers    total number of routers
%   I       interference matrix
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% interference matrix I (Iij = 1 if a direct or indirect interference is
% between clusters (routers) i and j (i.e. clusters i and j cannot be
% active at the same time), Iij=0 otherwise, Iii = 2.
I = [2,1,1,1,1,1;
     1,2,1,1,1,1;
     1,1,2,1,1,1;
     1,1,1,2,0,0;
     1,1,1,0,2,1;
     1,1,1,0,1,2];

% I = [2,1,1,1,1,1;
%      1,2,1,1,1,1;
%      1,1,2,1,1,1;
%      1,1,1,2,1,1;
%      1,1,1,1,2,1;
%      1,1,1,1,1,2];

% description of cluster-tree topology CTT (logical topology) -
% parent-child relation. Adjacency matrix (directed graph): CTTij=1 if node
% j is parent of node i. CTTij=0 otherwise. indexes: [routers,end_nodes]
CTT = [0,0,0,0,0,0,0,0,0,0,0,0,0,0;
       1,0,0,0,0,0,0,0,0,0,0,0,0,0;
       1,0,0,0,0,0,0,0,0,0,0,0,0,0;
       1,0,0,0,0,0,0,0,0,0,0,0,0,0;
       0,1,0,0,0,0,0,0,0,0,0,0,0,0;
       0,1,0,0,0,0,0,0,0,0,0,0,0,0;
       1,0,0,0,0,0,0,0,0,0,0,0,0,0;
       1,0,0,0,0,0,0,0,0,0,0,0,0,0;
       0,1,0,0,0,0,0,0,0,0,0,0,0,0;
       0,0,1,0,0,0,0,0,0,0,0,0,0,0;
       0,0,1,0,0,0,0,0,0,0,0,0,0,0;
       0,0,0,1,0,0,0,0,0,0,0,0,0,0;
       0,0,0,0,1,0,0,0,0,0,0,0,0,0;
       0,0,0,0,0,1,0,0,0,0,0,0,0,0];
 
%number of routers
no_routers = 6;

