function [p,A,W,dedication,proc_labels,task_labels,active_clusters,flow_links] = zig2tasks(ctf,cluster_proctime,interferences)
% ZigBee to tasks - convert routers/flows to processors/tasks
%
% PARAMETERS:
%   ctf     structure of cluster-tree flows
%   cluster_proctime
%   interferences
%
%   p               processing time
%   A               adjacency matrix constraining ofset start times
%   W               adjacency matrix constraining real start times               
%   dedication      dedication to given processors
%   proc_labels     names of processors corresponds with routers' names
%   task_labels     task_name in sequence
%   active_clusters
%   flow_links
%
% Copyright Hanzalek hanzalek@fel.cvut.cz
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%create dummy cluster
cluster_proctime(:,size(cluster_proctime,2)+1)=[0,0,0]';
%create dummy processor to host all dummy tasks
proc_names = [0]; 
%dummy source task for absolute deadlines (i.e. related to the start of the period)
task_names = [0,0,0]; 
p(1) = 0;
W = [-Inf];
A = [-Inf];
dedication{1} = [1];    %size(cluster_proctime,2);         

%create one real task for each cluster with nonzero proctime 
for cluster_id = 1:(size(cluster_proctime,2)-1)
    if any(cluster_proctime(:,cluster_id)~=[0,0,0]')
        %create new task
        task_names = [task_names;[cluster_id,cluster_id,cluster_id]];
        task_id(cluster_id) = size(task_names,1); %index of the task
        p(task_id(cluster_id)) = sum(cluster_proctime(:,cluster_id));
        W = [W,-Inf*ones(size(W,1),1);-Inf*ones(1,size(W,1)+1)];    %add empty column and row
        A = [A,-Inf*ones(size(A,1),1);-Inf*ones(1,size(A,1)+1)];    %add empty column and row
        %find dedication and create new processor if it does not exist
        dedication{task_id(cluster_id)}=[];
        for k=1:length(interferences{cluster_id})
            new_proc=interferences{cluster_id}(k);
            ii=1;
            proc_ex=0; 
            while ((ii<=length(proc_names))&&(proc_ex==0))
                if proc_names(ii,:)==new_proc
                    proc_ex=1;
                else
                    ii=ii+1;
                end
            end
            if proc_ex	%processor is already created
                dedication{task_id(cluster_id)}=[dedication{task_id(cluster_id)},ii];
            elseif new_proc==Inf   %dummy processor is put on the first solver processor
                dedication{task_id(cluster_id)}=[dedication{task_id(cluster_id)},1];
            else   %create new processor
                proc_names=[proc_names;new_proc];
                dedication{task_id(cluster_id)}=[dedication{task_id(cluster_id)},size(proc_names,1)];
            end
        end
    end
end
no_clusters=length(p);

%crreate tree of tasks for each flow
dummy_root = -Inf*ones(1,length(ctf));
for i=1:length(ctf)  %for each flow
     froots = find(ctf(i).father==0); %ID of a sink
     if isempty(froots)
         error('zigbee_tree:zig2tasks','There is no root!');
     elseif length(froots)>1 %several sinks (roots) -> from inforest make intree while adding dummy root
         dummy_root(i) = length(ctf(i).father)+1;
         ctf(i).father(froots) = dummy_root(i)*ones(length(froots),1);
         ctf(i).father(dummy_root(i)) = 0;
         ctf(i).rld(dummy_root(i)) = 0;
         ctf(i).processing_time(dummy_root(i)) = 0;
         ctf(i).dedication{dummy_root(i)} = Inf; %dummy root task is on the first physical processor
         ctf(i).r(dummy_root(i)) = 0;
         ctf(i).d_rel(dummy_root(i)) = Inf;
         ctf(i).task_names(1:3,dummy_root(i)) = [i;0;0];
     end
    	 [p,W,A,dedication,proc_names,task_names] = intree2tasks(p,W,A,dedication,proc_names,task_names,ctf(i),cluster_proctime,no_clusters);
end


%processors' labels
proc_labels(1)={num2str(0)};
for i=1:(size(proc_names,1))
    proc_labels(i)={['Clus',num2str(proc_names(i,1))]};
end

active_clusters = [];
for i=1:no_clusters-1 %dummy cluster
    active_clusters = [active_clusters, task_names(i+1,1)];
end

flow_links = [0,0,0]'; %[id,src,dest]'
%tasks' labels
task_labels(1)={'dumy_start'};
for i=2:no_clusters
    task_labels(i)={['c',num2str(task_names(i,2))]};
    flow_links = [flow_links, [0,0,0]'];
end

for i=(no_clusters+1):length(p)
    task_labels(i)={['id ',num2str(task_names(i,1)),',',num2str(task_names(i,2)),'->',num2str(task_names(i,3)),',T_{',num2str(i),'}']};
    ind=1+find(active_clusters==task_names(i,2));
    task_labels(ind)={[task_labels{ind},'f',num2str(task_names(i,1))]};
    flow_links = [flow_links, [task_names(i,1),task_names(i,2),task_names(i,3)]']; %[id,src,dest]'
end

