/*************************************************************************/
/* ILP model of periodic problem for ZigBee*/
/*************************************************************************/
/* odlisnost od gpr je v parametrech A (hrany s volitelnou vyskou) a D_A (aktivni procesor pro task, ostatni procesory v D jsou pasivni; dva tasky mohou pouzivat tentyz pasivni procesor)*/

/* number of tasks*/
param n, integer, >= 0;

/* number of processors*/
param m, integer, >= 0;

/* length of period*/
param per, integer, >= 0;

/* maximum number of execution periods*/
param qUB, integer, >= 0;

/*set of tasks*/
set T := 1..n;

/*set of processors*/
set P := 1..m;

/*processing time*/
param p{i in T};

/*Adjacency matrix constraining real start times*/
param W{i in T, j in T};

/*Adjacency matrix constraining ofset start times^*/
param A{i in T, j in T};

/*dedication*/
param D{i in T, j in P}, integer;

/*dedication active processor*/
param D_A{i in T}, integer;

/*x[i,j]=1 means that Ti preceedes Tj, only upper triangle is used, only for real tasks*/
var x{i in T, j in T}, binary;

/*s[i] stands for ofset start times^ of task Ti*/
var s{i in T}, >= 0, <= (per - 1);

/*q[i] stands for index of the execution period of task Ti*/
var q{i in T}, integer, >= 0, <= qUB;

/*makespan*/
var qmax, <= qUB;

/*print arcs */
/*printf{i in T, j in T: W[i,j] > -1e100}: " arc from T%d  to T%d is W[%d,%d]=%d \n",i,j,i,j,W[i,j];*/

/* conflict[i,j] = 1 iff T[i] has at least one common processor of single capacity with T[j] otherwisee conflict[i,j] = 0 */
param conflict{i in T, j in T} :=
   if exists{k in P} (D[i,k]>=1 and D_A[j]=k) then 1   
   else 0;

/* EDIT BY PJU */
/* printf{i in T, j in T: conflict[i,j]==1}: " common single processor for T%d  and T%d (conflict=%d)\n", i,j,conflict[i,j]; */


/****************************** ILP model ********************************/
/*objective */
/*minimize obj : qmax; */
/*minimize obj : sum{i in T} q[i]; */
minimize obj : sum{i in T} (per*q[i] + s[i]); 
/* minimize obj : 0; */

/*real precedence constraints*/
s.t. real_prec_con{i in T, j in T: i<>j and W[i,j] > -1e100}:
    s[j] + per*q[j] - s[i] - per*q[i] >= W[i,j];

/*precedence constraints*/
s.t. ofst_prec_con{i in T, j in T: i<>j and A[i,j] > -1e100}:
    s[j] - s[i] >= A[i,j];

/* processor constraints for real tasks only*/
s.t. proc_con_l{i in T, j in T: j>i and conflict[i,j]==1 and p[i]>0 and p[j]>0}:
     p[j]<=s[i]-s[j]+per*x[i,j];
s.t. proc_con_r{i in T, j in T: j>i and conflict[i,j]==1 and p[i]>0 and p[j]>0}:
     s[i]-s[j]+per*x[i,j]<=per-p[i];

/* OBJECTIVE OF THE OPTIMIZATION */
s.t. execution_periods_used{i in T}: q[i] <= qmax;

solve;

/*************************************************************************/
printf"\nResult of scheduling\n";

printf"Start times:\n";
printf{i in T}: "%d\n", s[i];

printf"Indeces of the execution period:\n";
printf{i in T}: "%d\n", q[i];

/*printf" Cmax is %d ticks\n", Cmax;*/





end;

